package de.mattv.fileserver.data;

import de.mattv.fileserver.Utils;
import lombok.Getter;
import de.mattv.fileserver.util.NonNull;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;

import java.time.Duration;
import java.time.Instant;
import java.util.Collection;
import java.util.List;

public class Token implements Authentication {
    private static final Duration LIFETIME = Duration.ofHours(24);
    private static final Duration LIFETIME_SRW = Duration.ofHours(1);

    private static final SimpleGrantedAuthority AUTHORITY_USER = new SimpleGrantedAuthority("ROLE_USER");
    private static final SimpleGrantedAuthority AUTHORITY_ADMIN = new SimpleGrantedAuthority("ROLE_ADMIN");

    private @NonNull Instant expiresAt;
    @Getter private final @NonNull String token;
    @Getter private final Long shareRoot;
    @Getter private final boolean shareCanRw;
    @Getter private final boolean shareRw;
    @Getter private @NonNull User user;
    @Getter private User sudoRealUser = null;

    public void refresh() { expiresAt = Instant.now().plus(shareRoot == null ? LIFETIME : LIFETIME_SRW); }
    public boolean isShare() { return shareRoot != null; }
    public boolean expired() { return Utils.instantExpired(expiresAt); }
    public boolean inSudo() { return sudoRealUser != null; }
    public boolean isAdmin() { return (sudoRealUser != null) || user.admin; }
    public User getRealUser() { return sudoRealUser != null ? sudoRealUser : user; }

    public void unSudo() {
        if (sudoRealUser != null)
            user = sudoRealUser;
        sudoRealUser = null;
    }

    public void sudo(@NonNull User newUser) {
        if (this.sudoRealUser == null)
            sudoRealUser = user;
        user = newUser;
    }

    public Token(@NonNull String token, @NonNull User user, Long shareRoot, boolean shareCanRw, boolean shareRw) {
        this.token = token;
        this.user = user;
        this.shareRoot = shareRoot;
        this.shareCanRw = shareCanRw;
        this.shareRw = shareRw;
        this.refresh();
    }

    @Override public Object getDetails() { return null; }
    @Override public String getName() { return String.valueOf(user.id); }
    @Override public Object getPrincipal() { return this; }
    @Override public Object getCredentials() { return this; }
    @Override public boolean isAuthenticated() { return true; }

    @Override
    public Collection<? extends GrantedAuthority> getAuthorities() {
        if (isShare()) return List.of();
        return isAdmin()
                ? List.of(AUTHORITY_USER, AUTHORITY_ADMIN)
                : List.of(AUTHORITY_USER);
    }

    @Override
    public void setAuthenticated(boolean isAuthenticated) throws IllegalArgumentException {
        throw new IllegalArgumentException("Cannot set authenticated");
    }

    @Override
    public String toString() {
        if (shareRoot == null) return "Token{user=" + user + ", real_user=" + sudoRealUser + '}';
        else return (shareRw ? "ShareRW(" : "Share(") + token + ")";
    }
}
