package de.mattv.fileserver.routes.auth;

import de.mattv.fileserver.Response;
import de.mattv.fileserver.data.Token;
import de.mattv.fileserver.security.TokenService;
import de.mattv.fileserver.util.AuthUser;
import de.mattv.fileserver.util.AutoCloseLock;
import de.mattv.fileserver.util.NonNull;
import de.mattv.fileserver.util.UserOrShareRestController;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

@UserOrShareRestController
public class ShareUnlock {
    @PostMapping("/share_unlock")
    private @NonNull Response<String> session(@AuthUser Token token, @RequestBody @NonNull String pw) {
        if (!token.isShare()) return Response.e("Not a share token");
        if (!token.isShareCanRw()) return Response.e("Share can not be unlocked");
        if (token.isShareRw()) return Response.e("Share already unlocked");

        var user = token.getUser();
        var root = token.getShareRoot();
        try (AutoCloseLock ignored = new AutoCloseLock(user.nodeLock.readLock())) {
            var node = user.nodes.get(root);
            if (node == null) return Response.e("Invalid share");
            if (node.shareWritePw == null) return Response.e("Share can not be unlocked");
            if (!AuthUtils.PW_ENCODER.matches(pw, node.shareWritePw)) return Response.e("Wrong password");
            return Response.o(TokenService.createToken(user, node));
        }
    }
}
