package de.mattv.fileserver.routes.fs;

import de.mattv.fileserver.data.Node;
import de.mattv.fileserver.data.Token;
import de.mattv.fileserver.data.User;
import de.mattv.fileserver.security.ShareService;
import de.mattv.fileserver.util.AuthUser;
import de.mattv.fileserver.util.AutoCloseLock;
import de.mattv.fileserver.util.NonNull;
import de.mattv.fileserver.util.UserRestController;
import jakarta.annotation.Nullable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.util.Optional;

@UserRestController
public class Share {
    @PostMapping("/fs/share")
    private @NonNull Optional<String> share(@AuthUser Token token, @RequestBody @NonNull long id) {
        User user = token.getUser();
        try (AutoCloseLock ignored = new AutoCloseLock(user.nodeLock.readLock())) {
            Node node = user.nodes.get(id);
            if (node == null) return Optional.empty();
            return Optional.of(ShareService.createShare(user, node));
        }
    }

    @PostMapping("/fs/unshare")
    private void path(@AuthUser Token token, @RequestBody @NonNull long id) {
        User user = token.getUser();
        try (AutoCloseLock ignored = new AutoCloseLock(user.nodeLock.writeLock())) {
            Node node = user.nodes.get(id);
            if (node != null)
                ShareService.removeShare(node);
        }
    }


    private record SetPwBody(@NonNull long id, @Nullable String password) {}
    @PostMapping("/fs/share_pw")
    private void setPw(@AuthUser Token token, @RequestBody @NonNull SetPwBody body) {
        User user = token.getUser();
        try (AutoCloseLock ignored = new AutoCloseLock(user.nodeLock.writeLock())) {
            Node node = user.nodes.get(body.id);
            if (node != null)
                ShareService.setPassword(node, body.password);
        }
    }
}
