package de.mattv.fileserver.security;

import de.mattv.fileserver.data.Data;
import de.mattv.fileserver.data.Node;
import de.mattv.fileserver.data.User;
import de.mattv.fileserver.routes.auth.AuthUtils;
import de.mattv.fileserver.util.NonNull;
import jakarta.annotation.Nullable;
import org.apache.commons.lang3.RandomStringUtils;
import org.apache.commons.lang3.tuple.Pair;

import java.util.concurrent.ConcurrentHashMap;

public class ShareService {
    private static final ConcurrentHashMap<String, Pair<User, Node>> SHARES = new ConcurrentHashMap<>();

    public static void init() {
        for (User user : Data.USERS.values()) {
            for (Node node : user.nodes.values()) {
                if (node.shareString != null) {
                    SHARES.put(node.shareString, Pair.of(user, node));
                }
            }
        }
    }

    public static @Nullable Pair<User, Node> getShare(@NonNull String token) {
        return SHARES.get(token);
    }

    public static String createShare(@NonNull User user, @NonNull Node node) {
        if (node.shareString != null)
            return node.shareString;

        String name = RandomStringUtils.random(TokenService.SHARE_TOKEN_LENGTH, true, true);
        SHARES.put(name, Pair.of(user, node));
        node.shareString = name;
        return name;
    }

    public static void setPassword(@NonNull Node node, @Nullable String password) {
        if (node.shareString == null) return;
        node.shareWritePw = (password == null)
                ? null
                : AuthUtils.PW_ENCODER.encode(password);
        TokenService.removeShareTokens(node.id);
    }

    public static void removeShare(@NonNull Node node) {
        if (node.shareString == null) return;
        SHARES.remove(node.shareString);
        node.shareString = null;
        node.shareWritePw = null;
        TokenService.removeShareTokens(node.id);
    }
}
